﻿Imports System.IO
Imports SnippsCode.dsSnippsCode
Imports System.IO.Compression
Public Class SnippFunction
    'Deklaration
    Private DatPath As String = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData)
    Private DataFile As New FileInfo(DatPath & "\SnippsCode.snp")
    Private Property _Counter As Integer = My.Settings.BackupCounter
    Public Property _BackupSuccess As Boolean
    Public _NotImportedList As New List(Of String)

    'Load/Save
    Public Sub Load(ByVal myDS As DataSet)
        myDS.Clear()
        If DataFile.Exists Then
            myDS.ReadXml(DataFile.FullName)
        End If
    End Sub
    Public Sub Save(ByVal myDS As DataSet, Optional withCompression As Boolean = False, Optional myFile As FileInfo = Nothing)
        If myFile Is Nothing Then myFile = DataFile
        If withCompression Then
            Using fs As FileStream = New FileStream(DatPath & "\SnippsCode_" & _Counter & ".zip", FileMode.Create), zip = New ZipArchive(fs, ZipArchiveMode.Create)
                Dim zipEntry As ZipArchiveEntry = zip.CreateEntry(myFile.Name)
                Using writer As StreamWriter = New StreamWriter(zipEntry.Open())
                    myDS.WriteXml(writer)
                    _Counter += 1
                    My.Settings.BackupCounter = _Counter
                    _BackupSuccess = True
                End Using
            End Using
        Else
            myDS.WriteXml(myFile.FullName)
        End If
        myDS.AcceptChanges()
    End Sub
    Public Sub Reload(ByVal myDS As DataSet)
        Save(myDS)
        Load(myDS)
    End Sub
    Public Sub checkStart()
        If DataFile.Exists Then
            My.Settings.firstStart = False
        Else
            My.Settings.firstStart = True
        End If
    End Sub
    'Import XML
    Private Shared Function RemoveNS(xElmt As XElement) As XElement
        'Eliminiert den Namespace (e.g. xmlns="http://tempuri.org/xxx.xsd")
        Return New XElement(xElmt.Name.LocalName, If(xElmt.HasElements, xElmt.Elements().Select(Function(el) RemoveNS(el)), DirectCast(xElmt.Value, Object)))
    End Function
    Public Sub ExportToFile(ByVal theFile As String, ByVal txt As String)
        Using fs As New FileStream(theFile, FileMode.Create), writer As New StreamWriter(fs)
            writer.Write(txt)
        End Using
    End Sub
    Public Sub ImportFromFile(ByVal theFile As String)
        Using fNew As New frmNew
            fNew.bsNewProjektFolder.DataSource = Dts
            fNew.bsNewProjektFile.DataSource = Dts
            Dim pfRow = DirectCast(fNew.bsNewProjektFile.AddNew(), DataRowView)
            Dim pFile = DirectCast(pfRow.Row, ProjektDateiRow)
            Using fs As New FileStream(theFile, FileMode.Open), reader As New StreamReader(fs)
                pFile.Inhalt = reader.ReadToEnd()
            End Using
            pFile.ProjektOrdnerRow = Dts.ProjektOrdner(0)
            pFile.Name = Path.GetFileName(theFile)
            If fNew.ShowDialog() = DialogResult.OK Then
                fNew.bsNewProjektFile.EndEdit()
                Save(Dts)
            Else
                fNew.bsNewProjektFile.CancelEdit()
            End If
        End Using
    End Sub
    Public Sub ImportFromSnippsCode(ByVal snpFile As String)
        Dim _Imp As XElement = XElement.Load(snpFile)
        Dim _Import = RemoveNS(_Imp)
        Dim _ProjectFolder = From Cat In _Import.Elements("ProjektOrdner")
                             Select Cat.<Name>, Cat.<ID>
        Dim _ProjectFile = From Snip In _Import.Elements("ProjektDatei")
                           Select Snip.<Name>, Snip.<Inhalt>, Snip.<ProjektID>
        For Each project In _ProjectFolder
            If Not Dts.ProjektOrdner.ProjectExists(project.Name.Value) Then
                Dim parentRow = Dts.ProjektOrdner.AddProjektOrdnerRow(project.Name.Value)
                For Each projectfile In _ProjectFile
                    If projectfile.ProjektID.Value = project.ID.Value Then
                        Dts.ProjektDatei.AddProjektDateiRow(parentRow, projectfile.Name.Value, projectfile.Inhalt.Value)
                    End If
                Next
            Else
                _NotImportedList.Add(project.Name.Value)
            End If
        Next
    End Sub
    Public Sub ImportFromSnipps(ByVal oldXML As String)
        Dim _Imp As XElement = XElement.Load(oldXML)
        Dim _Import = RemoveNS(_Imp)
        Dim _ProjectFolder = From Cat In _Import.Elements("Category")
                             Select Cat.<Description>, Cat.<ID>
        Dim _ProjectFile = From Snip In _Import.Elements("Snippets")
                           Select Snip.<SnippetInfo>, Snip.<Snippet>, Snip.<CategoryID>
        For Each oldCat In _ProjectFolder
            If Not Dts.ProjektOrdner.ProjectExists(oldCat.Description.Value) Then
                Dim parentRow = Dts.ProjektOrdner.AddProjektOrdnerRow(oldCat.Description.Value)
                For Each oldSnip In _ProjectFile
                    If oldSnip.CategoryID.Value = oldCat.ID.Value Then
                        Dts.ProjektDatei.AddProjektDateiRow(parentRow, oldSnip.SnippetInfo.Value, oldSnip.Snippet.Value)
                    End If
                Next
            Else
                _NotImportedList.Add(oldCat.Description.Value)
            End If
        Next
    End Sub
End Class
